/*
 * @(#)src/classes/sov/javax/management/BadAttributeValueExpException.java, management, asdev 1.4
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 * IBM SDK, Java(tm) 2 Technology Edition, v5.0
 * (C) Copyright IBM Corp. 1998, 2005. All Rights Reserved
 * ===========================================================================
 */

/*
 * ===========================================================================
 (C) Copyright Sun Microsystems Inc, 1992, 2004. All rights reserved.
 * ===========================================================================
 */

 /*===========================================================================
 * Change activity:
 *
 *  Reason   Date  Origin    Description
 * -------- ------ ------    -----------------------------------------------------
 *  199097  160913 ggchaitanya  OSB:8019292 - Better Attribute Value Exceptions 
 * ===========================================================================
 */

/*
 * @(#)BadAttributeValueExpException.java	4.17 03/12/19
 * 
 */

package javax.management;

import java.io.IOException;                                      /*ibm@199097*/
import java.io.ObjectInputStream;                                /*ibm@199097*/

/**
 * Thrown when an invalid MBean attribute is passed to a query
 * constructing method.  This exception is used internally by JMX
 * during the evaluation of a query.  User code does not usually
 * see it.
 *
 * @since 1.5
 */
public class BadAttributeValueExpException extends Exception   { 


    /* Serial version */
    private static final long serialVersionUID = -3105272988410493376L;

    /**
     * @serial The attribute value that originated this exception
     */
    private Object val;

    /**
     * Constructs an <CODE>BadAttributeValueExpException</CODE> with the specified Object.
     *
     * @param val the inappropriate value.
     */
    public BadAttributeValueExpException (Object val) { 
        this.val = val == null ? null : val.toString();           /*ibm@199097*/
    } 
   
 
    /**
     * Returns the string representing the object.
     */
    public String toString()  { 
	return "BadAttributeValueException: " + val;
    } 

    /*ibm@199097 starts*/
    private void readObject(ObjectInputStream ois) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField gf = ois.readFields();
        Object valObj = gf.get("val", null);

        if (valObj == null) {
            val = null;
        } else if (valObj instanceof String) {
            val= valObj;
        } else if (System.getSecurityManager() == null
                || valObj instanceof Long
                || valObj instanceof Integer
                || valObj instanceof Float
                || valObj instanceof Double
                || valObj instanceof Byte
                || valObj instanceof Short
                || valObj instanceof Boolean) {
            val = valObj.toString();
        } else { // the serialized object is from a version without JDK-8019292 fix
            val = System.identityHashCode(valObj) + "@" + valObj.getClass().getName();
        }
    }
    /*ibm@199097 ends*/
 }
