/*
 * @(#)src/tools/pfm/java_md.c, tool, aadev, 20130113 1.86
 * ===========================================================================
 * Licensed Materials - Property of IBM
 * "Restricted Materials of IBM"
 *
 * IBM SDK, Java(tm) 2 Technology Edition, v5.0
 * (C) Copyright IBM Corp. 1998, 2010. All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ===========================================================================
 */

/*
 * ===========================================================================
 (C) Copyright Sun Microsystems Inc, 1992, 2004. All rights reserved.
 * ===========================================================================
 */




/*
 * ===========================================================================
 * Function:
 *     Platform specific code for the java launcher for AIX
 *
 * ===========================================================================
 */
/*
 * ===========================================================================
 * Change activity:
 *
 * Reason   Date   Origin   Description
 * ------   ----   ------   --------------------------------------------------
 * 041302   040302 kwb      Globalize the java launcher
 * 041977   010402 vijayg   Java sdk tools (e.g. javac) fail
 * 042617   100402 vijayg   Change value of totlen in setLibPath()
 * 042180.1 150402 vpatil   java alters child process's env
 * 051143   230502 r1msrini Compatibility for 64 bit addresses for 42180.1
 * 051559.1 180602 r1msrini Rewrite setLibPath to filter out Netscape libraries.
 * 0053141  110702 nmasson  Fix FindJVMDLL
 * 052427.1 190702 hvivek   Change the environment variable used by 42180.1
 * 054361.1 230802 prakashr Java corrupts environment if it has variables with emedded \n
 * 054875.1 240902 cheesemp Removed default option: MALLOCMULTIHEAP=1
 * 055462   260902 kwb      Map zh_TW and zh_MO to zh_TW
 * 054795.1 270902 srowland Add pid to _IBM_ENV_INITIAL
 * 056370   071102 kwb      Use iconv for zh_TW
 * 056786.1 281102 kstares  Remove AIXTHREAD_*_DEBUG env variable settings
 *			                as the system default is now "OFF" and we would
 *			                benefit from being able to turn them on occasionally
 * 056704.1 231202 kstares  warn when hard data ulimit limits setting data limit to unlimited
 * 052350   220103 srowland Enhance AIX Large Page Support / Memory usage
 * 057716.1 230103 asangiri Relative path cause BuildHomeDirectory to die
 * 057806.1 220103 MBluemel If the environment is more than a PIPE buffer full,
 *                          use a temporary file...
 * 61901.1  110703 r1msrini Child process not to inherit ENV variables
 * 064361   231203 pratunga Change AIX loader to locate correct libjvm on j9
 * 067741   010404 mbluemel PD Build support
 * 073951   280504 kiranram Change maxdata value setting for J9
 * 071326   230604 pratunga Produce correct -X usage text for J9
 * 082754   140205 cwhite   Environment variable fix
 * 083268   210205 cwhite   PCR1479: *AIX64* ulimit issue for 64bit Java
 * 083586   020305 cwhite   Refix for 83268
 * 087841   280405 andyt    Warnings in java_md.c
 * 088734   180505 nichanir Rename the launcher's java*.properties files to
 *                          launcher*.properties
 * 088699   190505 kaaruna  Upgrade to 5.0 launcher 
 * 090911   290605 madison  Use 64bit interface
 * 091094   140705 bygravec Search for IBM SDKs to satisfy -version:
 * 076310   210705 bygravec Remove J9J2SE ifdefs
 * 093221   260705 bygravec Missing strdup
 * 094058   090905 andyt    LDR_CNTRL=MAXDATA not being set by java command
 * 095333   120905 andyt    BLDBRK: j9ap32rtdev-20050912 Syntax error in java_md.c
 * 098079.1 061205 andyt    Multi-symlinks to executeable fails
 * 103063   060406 andyt    nodeagent throwing core at restart
 * 103292   070406 andyt    BLDBRK: jclap32dev-20060407a - compile errors
 * 102016   110406 smithwil libmawt.a loaded from wrong location
 * 103031   260606 kiranram Enhancements to GetXUsage
 * 107777   180806 andyt    RAD:Linux: unable to run javadoc if SDK  in directory named jre
 * 105998   191206 smithwil Fix a bug in the change for 102016
 * 165484   040510 nmasson  Get JRE directory correctly
 * 176206   040211 enasser  Ensure JavaJREDir set correctly when jre directory  
 *                          name is modified
 * 191063   260412 gayatrip JVM crash due to big process number
 * ===========================================================================
 */


#include "java.h"
#include <dlfcn.h>
#include <fcntl.h>                                                /*ibm@57806.1*/
#include <string.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <locale.h>
#include <langinfo.h>		/* For nl_langinfo */		     //ibm.56370
#include <dirent.h>							/*ibm@88699*/
#include "manifest_info.h"					/*ibm@88699*/
#include "version_comp.h"					/*ibm@88699*/
#if defined(IBM_AIX)
#include <iconv.h>						     //ibm.56370
#endif
/*ibm@52350 starts*/
#include <sys/utsname.h>
#include <unistd.h>
#if !defined(_SC_LARGE_PAGESIZE)
#define  _SC_LARGE_PAGESIZE 116
#endif /* _SC_LARGE_PAGESIZE */
/*ibm@52350 ends*/

//ibm@7468  begin
#if defined(IBM_LINUX)
/*
 *  libgen.h removed from Linux as it redefines basename to be
 *  __xpg_basename which gives incorrect results.  The prototypes for
 *  dirname and basename are instead defined here
 */
extern char *basename(const char *filename);
extern char *dirname(const char *path);
#define TRUE 1						    /*ibm@7468*/
#define FALSE 0						    /*ibm@7468*/
#else
#include <libgen.h>
#endif
//ibm.@7468  end
#include <sys/stat.h>					    /*ibm@106*/
#include <errno.h>					    /*ibm@7468*/

#define JNI_LIB_PREFIX		"lib"
#define SYS_LIBRARY_PATH	"LIBPATH"
#define JNI_LIB_SUFFIX		".so"  /*ibm@76310*/

#if defined(IBM_BUILD_TYPE_dev)
    #define JVM_DLL  JNI_LIB_PREFIX "jvm" JNI_LIB_SUFFIX   /*ibm@67741*/
#elif defined(IBM_BUILD_TYPE_col)
    #define JVM_DLL  JNI_LIB_PREFIX "jvm_g" JNI_LIB_SUFFIX
#else
    #define JVM_DLL  JNI_LIB_PREFIX "jvm" JNI_LIB_SUFFIX
#endif

static char * JavaHomeDir;     /* Home directory - sdk		   */
static char * JavaJREDir;      /* JRE directory  - sdk/jre/bin */
static char JavaLibDir[MAXPATHLEN];                   /* ibm@67741 */
static char * JavaJVMDir;      /* JVM directory  - sdk/jre/bin/classic */
static char * useIconv = NULL;					     //ibm.56370
static char * IBM_VERSIONS_FILE = "/jre/lib/ibmversions.txt"; /*ibm@91094*/

/* start ibm@1369
 * Expand command line arguments.
 */
void
ExpandCommandLineArgs() {
    /* DO NOTHING */
}
/* end   ibm@1369 */

/**************************************************************************
 * name		- ibmFindJVMDLL
 * description  -
 *      Find a JVM DLL.
 *
 * parameters   - none                                   ibm@176206
 * returns      - path to JVM DLL
 **************************************************************************/

static char *						     /*ibm@106*/
ibmFindJVMDLL() {                                   /*ibm@176206*/ 
    struct stat s;
    char   home[MAXPATHLEN];

    /* ibm@176206-starts */
#define JVMType "classic"                                      

    /*  
     * Look through various directories for the JVM DLL library.  
     */

    /*
     * Get the application JRE directory  to search for the JVM DLL.
     */

    strncpy(home, JavaJREDir, MAXPATHLEN-50); 
    home[MAXPATHLEN-50] = 0;
    strcat(home, "/" JVMType);
    strcat(home, "/" JVM_DLL);
    if (stat(home, &s) == 0) 	   
	    return strdup(home);	   

    /*
     * The JVM DLL was not found in the JRE directory. Check whether
     * it is present in the SDK bin directory.
     *
     * Get the application home directory  to search for the JVM DLL.
     */

    strncpy(home, JavaHomeDir, MAXPATHLEN-50); 
    home[MAXPATHLEN-50] = 0;
    strcat(home, "/bin"); 
    strcat(home, "/" JVMType);
    strcat(home, "/" JVM_DLL);
    if (stat(home, &s) == 0) 	   
	    return strdup(home);	   

    /* ibm@176206-ends */

    /* JVM not found */
    return(char *)NULL;
}


/**************************************************************************
 * name	       - LoadJavaVM (ibm@149)
 * description -
 *      Load JVM and initialize the invocation functions.  Dynamically
 *      open the jvm shared library and find the entry points.
 *
 * parameters  - jvmtype (name of jvm type, currently "classic")
 *		 ifn (vector of functions which receives the function
 *		      pointers found in the jvm DLL)
 * returns     - JNI_TRUE if successful
 **************************************************************************/
jboolean
LoadJavaVM(char *jvmtype, InvocationFunctions *ifn) {
    char * jvmpath, * jp;
    void * hJvm;

    if (!JavaJVMDir) {					/*ibm@53141*/
	showMessage(stderr, "JVM.not.found", c2jc(JVM_DLL), NULL, 0);
	return JNI_FALSE;
    }
    jvmpath = MemAlloc(strlen(JavaJVMDir) + 50);
    strcpy(jvmpath, JavaJVMDir);
    strcat(jvmpath, "/"JVM_DLL);
    hJvm = dlopen(jvmpath,RTLD_LAZY);
    if (hJvm==NULL) {
	if (_launcher_debug) {							/*ibm@88699*/
	    fprintf(stderr, "LoadJavaVM: unable to open DLL (%s)\n", jvmpath);
	    fprintf(stderr, "%s\n", dlerror());
	}
	if (errno == ENAMETOOLONG) {
	    showMessage(stderr, "Path.too.long", c2jc(SYS_LIBRARY_PATH), NULL, 0);
	} else {
	    showMessage(stderr, "Error.loading", c2jc(jvmpath), NULL, 0);
	}
	free(jvmpath);
	return JNI_FALSE;
    }

    ifn->CreateJavaVM = (jint (JNICALL *)(JavaVM **pvm, void **penv, void *args))
			dlsym(hJvm,"JNI_CreateJavaVM");
    ifn->GetDefaultJavaVMInitArgs = (jint (JNICALL *)(void *args))
				    dlsym(hJvm,"JNI_GetDefaultJavaVMInitArgs");
/*ibm@76310..*/
    /* Start ibm@103031 */
    if ((ifn->GetXUsage_JVMAware = (jint (JNICALL *)(char *exename, JavaVMInitArgs *args))
                                      dlsym(hJvm,"GetXUsage_JVMAware")) == NULL) {
        ifn->GetXUsage = (jint (JNICALL *)(char *exename))
                     dlsym(hJvm,"GetXUsage");
    }
    /* End ibm@103031 */
/*..ibm@76310*/
    if (ifn->CreateJavaVM == 0 ||
	ifn->GetDefaultJavaVMInitArgs == 0) {
	showMessage(stderr, "JNI.interfaces.not.found", c2jc(jvmpath), NULL, 0);
	return JNI_FALSE;
    }
    free(jvmpath);
    return JNI_TRUE;
}

/**************************************************************************
 * name	       - GetXUsagePath
 * description -
 *      Get the path to the file that has the usage message for -X options.
 *
 * parameters  - buf (the receiving buffer for the path)
 *		 bufsize (the size of buf)
 * returns     -
 **************************************************************************/
void
GetXUsagePath(char *buf, jint bufsize) {
    *buf = 0;
}


/**************************************************************************
 * name	       - ResolveSymbol (ibm@7468)                       ibm@98079.1
 * description -
 *      Given a buffer containing a path resolve any symbols in the last
 *      component of the path. If the symbol resolves to a relative path
 *      add it to the original. Otherwise replace it.
 *
 * parameters  - pUsed (ptr to a buffer holding the current path)
 * returns     -
 *************************************************************************/
static void
ResolveSymbol(char *pUsed) {
    char buffer[MAXPATHLEN];

    if (realpath(pUsed, buffer)) {
        strcpy(pUsed, buffer);
    }
}

/**************************************************************************
 * name	       - BuildHomeDir (ibm@7468)
 * description -
 *      From the supplied executable name determine the true home directory.
 *      This function is called once to set up JavaHomeDir.
 *
 * parameters  - fname (The name of this executable, may be short or long)
 * returns     -
 *************************************************************************/
void
BuildHomeDir(char *fname) {
    char *path;
    char *p;
    char *p1;								/*ibm@57716.1*/
    int len;
    char dir[MAXPATHLEN];
    char *found = NULL;
    struct stat s;
    mode_t mode;							/*ibm@39752.1*/

    /*
     *  Is the exe name fully, or partially qualified?
     */
    if (strchr(fname, '/') != NULL && stat(fname, &s) == 0) {
	if (fname[0] == '/') {
	    strcpy(dir, fname);
	    found = dir;
	} else {
	    getcwd(dir, MAXPATHLEN);

	    path = dir;
	    p = fname;
	    while (p[0] == '.') {
		if (strstr(p, "../") == p) {
		    p = p+3;
		    p1 = dirname(path);					/*ibm@57716.1*/
                    if ( path != p1 )					/*ibm@57716.1*/
                         strcpy(path, p1);				/*ibm@57716.1*/
		} else {
		    p = p+2;
		}
	    }
	    strcat(path, "/");
	    strcat(path, p);

	    found = path;
	}
    } else {
	/*
	 * If not found, assume it's a short name and search the path variable...
	 */

	path = getenv("PATH");

	while (path != NULL) {
	    /*
	     *  Get the first element of the path
	     */

	    p = strchr(path, ':');

	    if (p != NULL) {

		/*
		 * There is more than one
		 */
		len = p - path;
		strncpy(dir, path, len);
		dir[len] = 0;
		path = p+1;
	    } else {

		/*
		 * Last path element...
		 */
		strcpy(dir, path);
		path = NULL;
	    }

	    /*
	     * Add the exe name to the path element and check it points to a file.
	     */
	    strcat(dir, "/");
	    strcat(dir, fname);
	    if (stat(dir, &s) == 0) {
		mode = s.st_mode & S_IFMT;				/*ibm@39752.1*/
		if (mode == S_IFREG) {					/*ibm@39752.1*/
		    /*
		     * We have found a match!
		     */
		    found = dir;
		    break;
		}							/*ibm@39752.1*/
	    }

	}

    }

    /*
     * Did we find it?
     */
    if (found != NULL) {
        char jvmDir[MAXPATHLEN];
	    /*
	     * Resolve symbolic link in name
         */
        ResolveSymbol(found);

	    /*
	     * Strip the exc's name off the end.
         */
        p = dirname(found);
        if (found != p) {
            strcpy(found, p);
        }

        if (strcmp(basename(found), "exe") == 0) {
            p = dirname(found);
            if (found != p) {
                strcpy(found, p);
            }
        }
        JavaJREDir = strdup(found);

	    /*
	     * Remove the next component which should be /bin
         */
        p = dirname(found);
        if (found != p) {
            strcpy(found, p);
        }

	    /*
	     * If the next component is /jre remove it
         */
        /* ibm@107777... */
        /* check this really is the jre dir (some tools are in sdk/bin) */
        /* and not an sdk in an application dir called jre */
        sprintf(jvmDir, "%s/bin/classic", found);
        /* ibm@107777 ibm@165484 */
         if (stat(jvmDir, &s) == 0) {                       /*ibm@176206*/    
            /* ...ibm@107777 */
            p = dirname(found);
            if (found != p) {
                strcpy(found, p);
            }
                                                           /*ibm@107777*/
        } else {			/*ibm@165484 start*/
            /* We have found the SDK directory instead of the JRE */
            free(JavaJREDir);
            JavaJREDir = malloc(strlen(found) + strlen("/jre/bin") + 1);
            strcpy(JavaJREDir, found);
            strcat(JavaJREDir, "/jre/bin");
        }				/*ibm@165484 end*/

        JavaHomeDir = strdup(found);

        if (_launcher_debug) {				/*ibm@88699*/
            fprintf(stderr, "JavaJREDir  = %s\n", JavaJREDir); /*ibm@165484*/
            fprintf(stderr, "JavaHomeDir = %s\n", JavaHomeDir);
        }
    }
}

/**************************************************************************
 * name	       - GetApplicationHome (ibm@7468)
 * description -
 *      Get the application home directory, which will have been calculated
 *      by BuildHomeDir.
 *
 * parameters  - buf     The receiving buffer for the path
 *		 bufsize The size of the receiving buffer (not used)
 * returns     - JNI_TRUE
 **************************************************************************/
jboolean
GetApplicationHome(char *buf, jint bufsize) {
    strncpy(buf, JavaHomeDir, bufsize-1); /*ibm@26075*/
    buf[bufsize-1] = 0;
    return JNI_TRUE;
}

/* Start of ibm@61901.1 */
int setAndRemember(char *varName,char *varValue) 
{
    int status = 0;                                                             /*ibm@82754*/
    char *oldSetting = 0;                                                       /*ibm@82754*/
    char *oldName = 0;                                                          /*ibm@82754*/
    char *oldValue;
    char *oldNameMask = "IBM_JVM_%s_OLD_VALUE";
    int oldNameLength;

    char *newSetting = 0;                                                       /*ibm@82754*/
    char *newName = 0;                                                          /*ibm@82754*/
    char *newNameMask = "IBM_JVM_%s_NEW_VALUE";
    int newNameLength;

    char *setting = 0;                                                          /*ibm@82754*/

    char *changedVars;
    char *changedVarsName = 0;                                                  /*ibm@82754*/
    char *changedVarsNameMask = "IBM_JVM_CHANGED_ENVVARS_%d";
    int changedVarsNameLength;

    char *newChangedVarsSetting = 0;                                            /*ibm@82754*/
    char *newChangedVars = 0;                                                   /*ibm@82754*/
    char *NOT_SET = "";
 
    oldValue = getenv(varName);
    if (oldValue != NULL) {                                                     /*ibm@82754 ...*/
        /*
         * We need to distinguish between (oldValue == NULL)
         * meaning the variable wasn't set at all, and
         * (strlen(oldValue) = 0) meaning it was set to
         * a null string, as these will differ in what we
         * need to do for our child processes (unset our
         * variable in the first case, set it to "" in the
         * second)
         */
        oldNameLength = strlen(oldNameMask) - 2 + strlen(varName);
        if ((oldName = malloc(oldNameLength + 1)) == NULL) { /* +1 for null terminator */
            status = -1;
            goto CleanupSetAndRemember;
        }
        sprintf(oldName,oldNameMask,varName);

        if ((oldSetting = malloc(oldNameLength + strlen(oldValue) + 2)) == NULL) { /* + 2 allows for "=" and null terminator */
            status = -1;
            goto CleanupSetAndRemember;
        }
        sprintf(oldSetting,"%s=%s",oldName,oldValue);
        status = putenv(strdup(oldSetting));
        if (status == -1) goto CleanupSetAndRemember;
    }                                                                           /*... ibm@82754*/
    
    newNameLength = strlen(newNameMask) - 2 + strlen(varName);
    if ((newName = malloc(newNameLength + 1)) == NULL) { /* +1 for null terminator */ /*ibm@82754*/
        status = -1;                                                            /*ibm@82754*/
        goto CleanupSetAndRemember;                                             /*ibm@82754*/
    }                                                                           /*ibm@82754*/
    sprintf(newName,newNameMask,varName);
    
    if ((newSetting = malloc(newNameLength + strlen(varValue) + 2)) == NULL) { /* + 2 allows for "=" and null terminator */ /*ibm@82754*/
        status = -1;                                                            /*ibm@82754*/
        goto CleanupSetAndRemember;                                             /*ibm@82754*/
    }                                                                           /*ibm@82754*/
    sprintf(newSetting,"%s=%s",newName,varValue);
    status = putenv(strdup(newSetting));                                        /*ibm@82754*/
    if (status == -1) goto CleanupSetAndRemember;                               /*ibm@82754*/

    if ((setting = malloc(strlen(varName) + strlen(varValue) + 2)) == NULL) { /* + 2 allows for "=" and null terminator */ /*ibm@82754*/
        status = -1;                                                            /*ibm@82754*/
        goto CleanupSetAndRemember;                                             /*ibm@82754*/
    }                                                                           /*ibm@82754*/
    sprintf(setting,"%s=%s",varName,varValue);
    status = putenv(strdup(setting));                                           /*ibm@82754*/
    if (status == -1) goto CleanupSetAndRemember;                               /*ibm@82754*/

    changedVarsNameLength = strlen(changedVarsNameMask) - 2 + 8;                //ibm@191063
    /* allow for a 8-digit pid */		//ibm@191063
    if ((changedVarsName = malloc(changedVarsNameLength + 1)) == NULL) { /*  +1 for null... */ /*ibm@82754*/
        status = -1;                                                            /*ibm@82754*/
        goto CleanupSetAndRemember;                                             /*ibm@82754*/
    }                                                                           /*ibm@82754*/
    sprintf(changedVarsName,changedVarsNameMask,getpid());

    changedVars = getenv(changedVarsName);
    if (changedVars == NULL) {  
       newChangedVars = strdup(varName);
    } else {
        newChangedVars = malloc(strlen(changedVars) + strlen(varName) +2);
        /* +2 allows for "," as a delimiter and null as a terminator */
        if (newChangedVars != NULL) sprintf(newChangedVars,"%s,%s",changedVars,varName); /*ibm@82754*/
    }
    if (newChangedVars == NULL) {                                               /*ibm@82754*/
        status = -1;                                                            /*ibm@82754*/
        goto CleanupSetAndRemember;                                             /*ibm@82754*/
    }                                                                           /*ibm@82754*/

    if ((newChangedVarsSetting = malloc(strlen(changedVarsName) + strlen(newChangedVars) + 2)) == NULL) { /* + 2 allows for "=" and null terminator */ /*ibm@82754*/
        status = -1;                                                            /*ibm@82754*/
        goto CleanupSetAndRemember;                                             /*ibm@82754*/
    }                                                                           /*ibm@82754*/
    sprintf(newChangedVarsSetting,"%s=%s",changedVarsName,newChangedVars);
    status = putenv(strdup(newChangedVarsSetting));

CleanupSetAndRemember:
    if (oldValue != NULL) {                                                     /*ibm@82754 ...*/
        free(oldName);
        free(oldSetting);
    }                                                                           /*... ibm@82754*/
    free(newName);
    free(newSetting);
    free(setting);
    free(changedVarsName);
    free(newChangedVarsSetting);
    free(newChangedVars);

    return status;                                                              /*ibm@82754*/
}
/* End of ibm@61901.1 */

/**************************************************************************
 * name	       - setLibPath (ibm@7468)
 * description -
 *      Clean up the LD_LIBRARY_PATH environment variable, and add our bin
 *      directory to it.
 *
 * parameters  - jvmtype: String (crucial value is "sovPd") ibm@67741
 * returns     -
 **************************************************************************/
void
setLibPath(char *jvmtype) { /*ibm@67741*/

    char * newpath;
    char *p;
    char *path = getenv(SYS_LIBRARY_PATH);
    int len;
    int totlen;
    char * path_segment; /* one of the directories in a path ibm@102016 */
    int path_segment_memalloc_successful = FALSE;                     /*ibm@105998*/

    if (!path) path = "";
    totlen = strlen(path)+strlen(JavaHomeDir)*3+100; /* ibm@42617 */
    newpath = MemAlloc(totlen);

    /*
     * Add the jre/bin directory to the start of LD_LIBRARY_PATH
     */
    strcpy(newpath, JavaLibDir);                    /*ibm@67741*/               /*ibm@82754*/
    if (strcmp(JavaLibDir, JavaJVMDir)) {
	strcat(newpath, ":");
	strcat(newpath, JavaJVMDir);
	strcat(newpath, ":");                      /*ibm@41977 */
	strcat(newpath, JavaHomeDir);              /*ibm@41977 */
        if (strcmp(jvmtype,"sovPd") != 0) {        /*ibm@67741*/
            strcat(newpath, "/jre/bin");           /*ibm@41977*/
        } else {                                   /*ibm@67741*/
            strcat(newpath, "/jre/pd/bin");        /*ibm@67741*/
        }                                          /*ibm@67741*/
    }

    /*
     * Now clean up LD_LIBRARY_PATH
     */
    while (path != NULL) {

	/* Get the first element of the path */

	p = strchr(path, ':');

	if (p == path) {
	    /* Discard null path entry "::" */
	    p++;
	} else {
	    if (p != NULL) {
		len = p - path;
		p++;
	    } else {
		len = strlen(path);
	    }

	    /* Discard trailing path separator */
	    if (len > 0 && path[len-1] == '/') {
		len--;
	    }

            /* ibm@102016 */
            /* check for "headless", "motif21" or "xawt" in this path segment */
            /* and discard any path segments that contain these strings */
            path_segment = MemAlloc(len + 1);
            if (path_segment != NULL) {
                /* set a flag for MemFree later ibm@105998 */
                path_segment_memalloc_successful = TRUE;              /*ibm@105998*/
                /* copy the first path segment in to the buffer */
                strncpy(path_segment, path, len);
                /* set null at end of string */
                path_segment[len] = 0;
            } else {
                path_segment = "";
            }

            if (strncmp(path, "/lib", 4) &&
                strncmp(path, "/usr/lib", 8) &&
                strncmp(path, "/usr/ccs/lib", 12) && len > 0 &&
                (strstr(path, "netscape/communicator") == NULL) &&      /*ibm@51559.1*/
                (strstr(path_segment, "headless") == NULL) &&           /*ibm@102016*/
                (strstr(path_segment, "xawt") == NULL) &&               /*ibm@102016*/
                (strstr(path_segment, "motif21") == NULL)) {            /*ibm@102016*/
                strcat(newpath, ":");
                strncat(newpath, path, len);
            }

            if (path_segment_memalloc_successful == TRUE) {           /*ibm@105998*/
                MemFree(path_segment);                                  /*ibm@102016*/
            }

	}

	/* Move to the next path item */
	path = p;

    }
    if (setAndRemember(SYS_LIBRARY_PATH,strdup(newpath)) != 0) {                /*ibm@82754...*/
        fprintf(stderr,"Environment variable setting for %s failed, errno: %d %s",
                SYS_LIBRARY_PATH, errno, strerror(errno));
    }                                                                           /*...ibm@82754*/
    MemFree(newpath);
}

/**************************************************************************
 * name	       - setEnvVars (ibm@6796)
 * description -
 *      Set the environment variables needed by Java.
 *
 * parameters  -
 * returns     -
 **************************************************************************/
void
setEnvVars() {
    char *p;			/*ibm@61901.1*/
#ifdef IBM_AIX
    p = getenv("AIXTHREAD_SCOPE");
    if (!(p && !strcmp(p, "S"))) {
        if (setAndRemember("AIXTHREAD_SCOPE","S") != 0) { /*ibm@8392*/ /*ibm@61901.1*/ /*ibm@82754...*/
            fprintf(stderr,"Environment variable setting for %s failed, errno: %d %s",
                    "AIXTHREAD_SCOPE", errno, strerror(errno));
        }                                                                           /*...ibm@82754*/
    }

    /* IBM@56786.1 Removed AIXTHREAD_*_DEBUG variables                    */ 

#if defined(__64BIT__)
    putenv("NULLPTR=NOSEGV");					/*ibm@24449*/
#endif
#endif

}

/**************************************************************************
 * name        - StoSizeT (ibm@52350)
 * description -
 *        Convert a string of the type nnnnn<K|k|M|m|G|g> to a scaler
 *        representing that number.
 *        where:
 *          K|k = kilobytes (1024)
 *          M|m = megabytes (1024*1024)
 *          G|g = gigabytes (1024*1024*1024)
 *
 * parameters  - string specifiying a size
 * returns     - size_t containing specified size
 **************************************************************************/
static size_t
StoSizeT (char *s)
{
    size_t n;

    n = strtoul(s, &s, 10);

    switch (*s) {
    case 'G': case 'g':
        n *= (1024*1024*1024);
        break;
    case 'M': case 'm':
        n *= (1024*1024);
        break;
    case 'K': case 'k':
        n *= (1024);
        break;
    case '\0':
        break;
    default:
        return 0;
    }
    return n;
}

/**************************************************************************
 * name	       - setLibAndExec (ibm@7468)
 * description -
 *      AIX and Linux dlopen have problems with LD_LIBRARY_PATH changes
 *      made in the same process that is calling dlopen. Both use the
 *      value of LD_LIBRARY_PATH at process exec time. If JAVA has been
 *      executed without LD_LIBRARY_PATH containing the java lib paths,
 *      this function adds them, and re-execs JAVA to ensure that dlopen
 *      sees the new LD_LIBRARY_PATH setting.
 *
 * parameters  - argc (The count of arguments passed to main)
 *		 arcv (Array of parameters taken from the command line)
 * returns     -
 **************************************************************************/
void
setLibAndExec(int argc, char *argv[], char * jvmtype) {
    char *p, *path;
    char *buffer;
    char * jvmpath, * jp;
    int reExecNeeded = FALSE;                                                   /*ibm@83268*/
    int lib1 = FALSE, lib2 = FALSE;
    int libfound;
#ifdef IBM_AIX
    struct rlimit64 limit;
    /*ibm@52350 starts*/
    int largePageToBeUsed = FALSE;
    size_t HeapSize=0,HeapSegs;
    static char ldr_cntrl[]="LDR_CNTRL=MAXDATA=0x80000000";
    static const char nums[]="012345678";
    size_t DataSegs;
    struct utsname name;
    /*ibm@52350 ends*/
#endif

    char *oldLdrCntrl = 0;                                                             /*ibm@94058,ibm@95333*/
    int i=0;                                                                           /*ibm@103292*/

    /*
     * Duplicate this here to make sure that we get the debug output
     * before the LIBPATH is set up.
     */
    if (getenv("_JAVA_LAUNCHER_DEBUG") != 0) {
	_launcher_debug = JNI_TRUE;			/*ibm@88699*/
    }

    /*
     * Find the application home dir from the application name
     */
    BuildHomeDir(argv[0]);

    //ibm.41302  begin
    /*
     * Find the JVM
     */
    jvmpath = ibmFindJVMDLL();           /*ibm@176206*/
    if (!jvmpath)
	return;
    JavaJVMDir = strdup(jvmpath);
    jp = strrchr(JavaJVMDir, '/');
    if (jp) *jp=0;
    if (_launcher_debug)				/*ibm@88699*/
	fprintf(stderr, "JavaJVMDir  = %s\n", JavaJVMDir);
    //ibm.41302  end

    /*
     * Look through LIBPATH for the java libraries.  Ensure
     * the current java libraries are before other potentially
     * conflicting libraries.
     */

   /*ibm@67741 - hack "pd" into paths if necessary*/
    strcpy(JavaLibDir,JavaJREDir);
    if (strcmp(jvmtype,"sovPd") == 0) {
        p=strrchr(JavaLibDir,'/');
        strcpy(p,"/pd/bin");
    }

    p = getenv(SYS_LIBRARY_PATH);
    if (!p) p = "";
    buffer = strdup(p);
    path = buffer;
    while (path != NULL) {
	if ((p = strchr(path, ':')) != NULL) {
	    *p++ = 0;
	}
	if (!strcmp(path, JavaJVMDir)) {
	    lib1 = TRUE;
	    if (!strcmp(path, JavaLibDir))              /*ibm@67741*/
		lib2 = TRUE;
	} else if (!strcmp(path, JavaLibDir)) {         /*ibm@67741*/
	    lib2 = TRUE;
	} else {
	    break;
	}
	path = p;
    }
    free (buffer);

    if (!lib1 || !lib2) {
        reExecNeeded = TRUE;                                                    /*ibm@83268*/
        setLibPath(jvmtype);                           /*ibm@67741*/
    }

#ifdef IBM_AIX
    /*
     * If the user has not set the thread scope, or has not set it to 'S',
     * we must reset it. This will enable a user with the correct environment
     * to avoid an execvp, the same is true for the otehr flags below.
     */
    p = getenv("AIXTHREAD_SCOPE");				/*ibm@9197*/
    if (!(p && !strcmp(p, "S"))) {				/*ibm@9197*/
	reExecNeeded=TRUE;					/*ibm@9197*/                            /*ibm@83268*/
    }

    /* IBM 56786.1 Removed AIXTHREAD_*_DEBUG variables                    */

#endif


    /* This was moved in IBM@9197 but cannot be placed before tests above */
    /* or they will always see the correct settings		  IBM@8872*/
    setEnvVars();

#ifdef IBM_AIX
    /*
     * ibm@24999
     * This mechanism is implemented in AIX on both PowerPC and
     * IA64 and is related to the static linker binder option
     * -bmaxdata, but is more flexible because the user
     * may set their own environment variable value of number of
     * segments for private data that is used for malloc() memory
     * allocations including thread stacks and the default Java
     * heap.
     * ibm@29024
     * Using LDR_CNTRL=MAXDATA is needed in both 32-bit and 64-bit
     * to use its ability to override the user soft data limit.
     */

    #if !defined(__64BIT__) || defined(_ARCH_IA64)      /* ibm@32140*/

    /* If NOT PPC64 - Check for "LDR_CNTRL" */

    p = getenv("LDR_CNTRL");
    /*ibm@52350 starts*/
    if (p == NULL) {
    /*ibm@94058...*/
        /* LDR_CNTRL is not in the environment - needs setting */
        oldLdrCntrl = "";
    } else {
        if (strstr(p, "MAXDATA=") == NULL) {
            /* LDR_CNTRL is set but does not contain MAXDATA - need to add MAXDATA */
            oldLdrCntrl = p;
        } else {
            /* LDR_CNTRL is set and contains an existing MAXDATA - leave as is */
            oldLdrCntrl = 0;
        }
    }
    if (oldLdrCntrl != 0) {
        char *newMaxData;
        char *newLdrCntrl;
    /*...ibm@94058*/
        int status = 0;                                                                /*ibm@82754*/

        /*
         *  Parse parameters and look for -Xlp (Large Page) and maximum heap size.
         *  If -Xlp is specified, use sysconf to determine if system supports Large Page.
         */
        for (i=0;i<argc;i++){
            if(!strcmp(argv[i], "-Xlp")) {
                largePageToBeUsed =  (sysconf(_SC_LARGE_PAGESIZE) > 0);
            } else if (!strncmp(argv[i], "-Xmx", 4)) {
                HeapSize = StoSizeT(argv[i]+4);
            } else if (!strncmp(argv[i], "-mx", 3)) {
                HeapSize = StoSizeT(argv[i]+3);
            }
        }

        /*
         *  Convert bytes to segments
         */
        HeapSegs = (HeapSize + 0x0fffffff) >> (32-4);

        /*
         * We will use the Very Large Memory Model with
         * Dynamic Segment Allocation (DSA) as this provides more flexibility in
         * using available memory for mapping and malloc.
         * ibm@83268 - we now only support AIX 5.2 and later, hence we can use DSA.
         */
        if (HeapSegs > 12) {
            newMaxData = "MAXDATA=0@DSA";                          /*ibm@61901.1*/ /*ibm@82754*/ /*ibm@94058*/
        } else if (HeapSegs > 9) {
            newMaxData = "MAXDATA=0XB0000000@DSA";                 /*ibm@61901.1*/ /*ibm@82754*/ /*ibm@94058*/
        } else {
            newMaxData = "MAXDATA=0XA0000000@DSA";                 /*ibm@61901.1*/ /*ibm@82754*/ /*ibm@94058*/
        }
        /*ibm@94058...*/
        if (strlen(oldLdrCntrl) > 0) {
            /* add MAXDATA to existing LDR_CNTRL */
            newLdrCntrl = malloc(strlen(p)+1+strlen(newMaxData)+1);
            if (newLdrCntrl) {
                sprintf(newLdrCntrl, "%s@%s", p, newMaxData);
            } else {
                fprintf(stderr, "Couldn't allocate space to modify LDR_CNTRL\n");
                newLdrCntrl = p;   /* since we can't append MAXDATA we'll leave it as it was */
            }
        } else {
            /* LDR_CNTRL was not set */
            newLdrCntrl = newMaxData;
        }
        status = setAndRemember("LDR_CNTRL", newLdrCntrl);
        if ((strlen(oldLdrCntrl) > 0) && (newLdrCntrl != p)) {
            free(newLdrCntrl);
        }
        /*...ibm@94058*/
        
        if (status != 0) {                                                          /*ibm@82754...*/
            fprintf(stderr,"Environment variable setting for %s failed, errno: %d %s",
                    "LDR_CNTRL", errno, strerror(errno));
        }                                                                           /*...ibm@82754*/
        reExecNeeded = TRUE;                                                    /*ibm@83268*/
    }
    /*ibm@52350 ends*/

    #endif       /* ibm@32140*/

    /*
     * Attempt to set ulimit to unlimited for both 32bit and 64bit AIX
     * If this fails then we alert the user to a potential out of memory error.
     * Note that this is only done for AIX 64bit since for 32bit the LDR_CNTRL
     * governs the limit.
     * ibm@83268 
     */
    #if defined(__64BIT__)   /*ibm@33012*/
        getrlimit64(RLIMIT_DATA,&limit);                        /*ibm@32147*/
    if ((limit.rlim_cur) != RLIM64_INFINITY) {                  /*ibm@32147*/
        limit.rlim_cur = RLIM64_INFINITY;                       /*ibm@32147*/
        setrlimit64(RLIMIT_DATA,&limit);                        /*ibm@32147*/
        getrlimit64(RLIMIT_DATA,&limit);           /* start of ibm@56704.1 */
        if ((limit.rlim_cur) != RLIM64_INFINITY) {
            if (!reExecNeeded) {
                fprintf(stderr,"WARNING: JVM is running with hard ulimit not set to unlimited, out of memory errors may occur\n");
                fprintf(stderr,"         If an out of memory error occurs try running \"chuser data_hard=-1 username\" as root\n");
                fprintf(stderr,"         before running java\n");
            } else {
                reExecNeeded=TRUE;
            }    
        }                                            /* end of ibm@56704.1 */    
    }                                                           /*ibm@32147*/
    #endif                   /*ibm@33012*/
#endif    /* IBM_AIX */

    if (reExecNeeded) {      /*ibm@9197,ibm@32147*/                             /*ibm@83268*/
	execvp(argv[0], argv);					/*ibm@9197*/
    }								/*ibm@9197*/
}


/*
 * Parse a platform name to separate the path and the file
 */
char * PlatformName(char * in, char * * cpath) {
    *cpath = ".";
    if (in[0]=='/') {
	char nbuf[PATH_MAX+1], * nptr;
	if (realpath(in, nbuf)) {
	    char * out = strdup(nbuf);
	    nptr = strrchr(out, '/');
	    if (out < nptr) {
		*cpath = out;
		*nptr = 0;
		return nptr+1;
	    }
	}
    }
    return in;
}


/*
 * Show a Unicode string.  This assumes that the wide char encoding
 * is in Unicode.  This is true for most current Unix systems.
 */
void ShowUnicode(FILE * f, jchar * str, int len) {
//ibm.56370  begin
    int     outlen = len*4;
    char  * sbuf = MemAlloc(outlen + 1);
    char  * sp = NULL;
    int     i, rc;

#if defined(IBM_AIX)
    if (useIconv) {
	iconv_t ic;
	char * outbuf, * inbuf;
	size_t outleft, inleft;

	ic = iconv_open(useIconv, "UCS-2");
	if (ic) {
	    outbuf = sbuf;
	    inbuf  = (char *)str;
	    inleft = len*2;
	    outleft = outlen;
	    if (!iconv(ic, &inbuf, &inleft, &outbuf, &outleft)) {        /*ibm@87841*/
		sp = outbuf;
	    }
	    iconv_close(ic);
	}
    }
    if (!sp)
#endif
//ibm.56370  end
    {
	sp = sbuf;
	for (i=0; i<len; i++) {
	    rc = wctomb(sp, (wchar_t)str[i]);
	    if (rc>0)
		sp += rc;
	}
    }
    *sp = 0;
    fwrite(sbuf, 1, (sp-sbuf), f);
    fflush(f);
    free(sbuf);
}


/*
 * Get the message file for this locale
 */
char * GetMessageFile(char *buf, jint bufsize) {
    char * cp;
    char   langname[8];
    char   ctryname[8];
    char * lc, * lp;

    /* Get the lang and country strings */
    lc = setlocale(LC_CTYPE, "");
    if (!lc) lc = "en";
    strncpy(langname, lc, 7);
    langname[7] = 0;
    ctryname[0] = 0;
    cp = strchr(langname, '_');
    if (cp) {
	*cp = 0;
	strcpy(ctryname, cp+1);
	ctryname[2] = 0;
    }
    //ibm.56370  begin
#ifdef IBM_AIX
    {
	char * p = nl_langinfo(CODESET);
	if (!strcmp(p, "IBM-eucTW")) {
	    useIconv = p;
	}
    }
#endif

    /* lower case the language name */
    lp = langname;
    while (*lp) {
	*lp++ = tolower(*lp);
    }
    //ibm.56370  end

    /* Get the directory one up from that containing the jvm.dll */
    if ((strlen(JavaJREDir)+24) > bufsize) {
	return NULL;
    }

    strcpy(buf, JavaJREDir);
    cp = buf+strlen(buf);
    strcpy(cp, "/launcher_");         /*ibm@88734*/
    cp += 10;                         /*ibm@88734*/

    /* Try for launcher_ll_CC.properties */
    if (*ctryname) {
	strcpy(cp, langname);
	strcat(cp, "_");
	strcat(cp, ctryname);
	strcat(cp, ".properties");
	if (!access(buf, 4))
	    return buf;
    }

    //ibm.55562  begin
    /* Special case to map traditional chinese */
    if (!strcmp(langname, "zh") &&
	(!strcmp(ctryname, "HK") || !strcmp(ctryname, "MO"))) {
	strcpy(cp, "zh_TW.properties");
	if (!access(buf, 4))
	    return buf;
    }
    //ibm.55562  end

    /* Try for launcher_ll.properties */       /*ibm@88734*/
    if (*langname) {
	strcpy(cp, langname);
	strcat(cp, ".properties");
	if (!access(buf, 4))
	    return buf;
    }

    /* Try for launcher.properties */          /*ibm@88734*/
    strcpy(cp-1, ".properties");
    if (!access(buf, 4))
	return buf;

    return 0;
}

/*ibm@88699 start*/
static const char *system_dir	= "/usr";
/*static const char *user_dir	= "/java";*/
static char *SetExecname(char **argv);
/*ibm@88699 end*/

/*  ibm@88699
 *	Determine if there is an acceptable JRE in the directory dirname.
 *	Upon locating the "best" one, return a fully qualified path to
 *	it. "Best" is defined as the most advanced JRE meeting the
 *	constraints contained in the manifest_info. If no JRE in this
 *	directory meets the constraints, return NULL.
 *
 *	Note that we don't check for errors in reading the directory
 *	(which would be done by checking errno).  This is because it
 *	doesn't matter if we get an error reading the directory, or
 *	we just don't find anything interesting in the directory.  We
 *	just return NULL in either case.
 *
 *	The historical names of j2sdk and j2re were changed to jdk and
 *	jre respecively as part of the 1.5 rebranding effort.  Since the
 *	former names are legacy on Linux, they must be recognized for
 *	all time.  Fortunately, this is a minor cost.
 */
static char
*ProcessDir(manifest_info *info, char *dirname)
{
    DIR64	    *dirp;       /*ibm@90911*/
    struct dirent64 *dp;         /*ibm@90911*/
    char    *best = NULL;
    int     offset;
    int	    best_offset = 0;
    char    *ret_str = NULL;
    char    buffer[PATH_MAX];
	char 	szTemp[PATH_MAX];

    if ((dirp = opendir64(dirname)) == NULL)   /*ibm@90911*/
	return (NULL);

    do {
        if ((dp = readdir64((DIR64 *)dirp)) != NULL) {   /*ibm@90911*/
            offset = 0;
            if ((strncmp(dp->d_name, "jre", 3) == 0) ||
                (strncmp(dp->d_name, "jdk", 3) == 0))
                offset = 3;
            else if (strncmp(dp->d_name, "j2re", 4) == 0)
                offset = 4;
            else if (strncmp(dp->d_name, "j2sdk", 5) == 0)
	        offset = 5;
            if (offset > 0) {
    	        if ((acceptable_release(dp->d_name + offset,
	            info->jre_version)) && CheckSanity(dirname, dp->d_name))
    	            if ((best == NULL) || (exact_version_id(
	                dp->d_name + offset, best + best_offset) > 0)) {
		        if (best != NULL)
		            free(best);
		        best = strdup(dp->d_name);
		        best_offset = offset;
                    }
            }
        }
    } while (dp != NULL);
    (void) closedir64((DIR64 *)dirp);     /*ibm@90911*/
    if (best == NULL)
	return (NULL);
    else {
	ret_str = MemAlloc(strlen(dirname) + strlen(best) + 2);
	ret_str = strcat(strcat(strcpy(ret_str, dirname), "/"), best);
	free(best);
	return (ret_str);
    }
}

/*  ibm@88699
 *	This is the global entry point. It examines the host for the optimal
 *	JRE to be used by scanning a set of directories.  The set of directories
 *	is platform dependent and can be overridden by the environment
 *	variable JAVA_VERSION_PATH.
 *
 *	This routine itself simply determines the set of appropriate
 *	directories before passing control onto ProcessDir().
 */
char*
LocateJRE(manifest_info* info)
{
    char    *path;
    char    *home;
    char    *target = NULL;
    char    *dp;
    char    *cp;
    /*ibm@91094..*/
    char    filepath[MAXPATHLEN];
    FILE    *ibmfile;
    char    buf[MAXPATHLEN+10];
    char    *ibmdir;
    char    *ibmversion;

    /* Open the properties file and check for a matching JRE
     * If a match is found, return it, else drop through to
     * Sun default behaviour
     */
    GetApplicationHome(filepath, MAXPATHLEN);
    strncat(filepath, IBM_VERSIONS_FILE, MAXPATHLEN - strlen(filepath));
    if ((ibmfile = fopen(filepath, "r")) != NULL) {
        if (_launcher_debug)
            printf("Parsing %s\n", filepath);
        fgets(buf, MAXPATHLEN + 10, ibmfile);
        while (!feof(ibmfile)) {
            ibmversion = strtok(buf, "=");
            if (ibmversion != NULL &&
                acceptable_release(ibmversion, info->jre_version)) {
                ibmdir = strtok(NULL, "\n");
                if (ibmdir != NULL && CheckSanity(ibmdir, "/")) {
                    if (_launcher_debug)
                        printf("Found %s\n", ibmdir);
                    return strdup(ibmdir);    /*ibm@93221*/
                }
            }
            fgets(buf, MAXPATHLEN + 10, ibmfile);
        }
        fclose(ibmfile);
    } else {
        if (_launcher_debug)
            printf("File %s not found\n", filepath);
        // no file found, so drop through to Sun default behaviour
    }

    /*
     * If we are still here, try getting JAVA_VERSION_PATH
     */
    /*..ibm@91094*/
    if (info->jre_restrict_search)
	path = strdup(system_dir);
    else if ((path = getenv("JAVA_VERSION_PATH")) != NULL)
	path = strdup(path);
    else
	if ((home = getenv("HOME")) != NULL) {
	    path = (char *)MemAlloc(strlen(home) + 13);
	    path = strcat(strcat(strcpy(path, home),":"), system_dir);
	} else
	    path = strdup(system_dir);

    /*
     * Step through each directory on the path. Terminate the scan with
     * the first directory with an acceptable JRE.
     */
    cp = dp = path;
    while (dp != NULL) {
	cp = strchr(dp, (int)':');
	if (cp != NULL)
	    *cp = (char)NULL;
	if ((target = ProcessDir(info, dp)) != NULL)
	    break;
	dp = cp;
	if (dp != NULL)
	    dp++;
    }
    free(path);
    return (target);
}

/* ibm@88699
 * Given a path to a jre to execute, this routine checks if this process
 * is indeed that jre.  If not, it exec's that jre.
 *
 * We want to actually check the paths rather than just the version string
 * built into the executable, so that given version specification (and
 * JAVA_VERSION_PATH) will yield the exact same Java environment, regardless
 * of the version of the arbitrary launcher we start with.
 */
void
ExecJRE(char *jre, char **argv)
{
    char    wanted[PATH_MAX];
    char    *execname;
    char    *progname;
    size_t  wanted_len;  /*ibm@91094*/

    /*
     * Resolve the real path to the directory containing the selected JRE.
     */
    if (realpath(jre, wanted) == NULL) {
	fprintf(stderr, "Unable to resolve %s\n", jre);
	exit(1);
    }

    /*
     * Resolve the real path to the currently running launcher.
     */
    execname = SetExecname(argv);
    if (execname == NULL) {
	fprintf(stderr, "Unable to resolve current executable\n");
	exit(1);
    }

    /*
     * If the path to the selected JRE directory is a match to the initial
     * portion of the path to the currently executing JRE, we have a winner!
     * If so, just return.
     */
    if (strncmp(wanted, execname, strlen(wanted)) == 0)
	return;			/* I am the droid you were looking for */

    /*
     * If this isn't the selected version, exec the selected version.
     */
#ifdef JAVA_ARGS  /* javac, jar and friends. */
    progname = "java";
#else             /* java, oldjava, javaw and friends */
#ifdef PROGNAME
    progname = PROGNAME;
#else
	{
		char *s;
		progname = *argv;
		if ((s = strrchr(progname, FILE_SEPARATOR)) != 0) {
			progname = s + 1;
		}
	}
#endif /* PROGNAME */
#endif /* JAVA_ARGS */

    /*
     * This should never happen (because of the selection code in SelectJRE),
     * but check for "impossibly" long path names just because buffer overruns
     * can be so deadly.
     */
    if (strlen(wanted) + strlen(progname) + 10 > PATH_MAX) {
        fprintf(stderr, "Path length exceeds maximum length (PATH_MAX)\n");
        exit(1);
    }

    /*
     * Construct the path and exec it.
     */
    /*ibm@91094..*/
    wanted_len = strlen(wanted);
    (void)strcat(strcat(wanted, "/bin/"), progname);
    if (access(wanted, X_OK) != 0) {
        *(wanted+wanted_len) = '\0';
        (void)strcat(strcat(wanted, "/jre/bin/"), progname);
    }
    argv[0] = wanted;
    /*..ibm@91094*/
    if (_launcher_debug) {							/*ibm@88699*/
        int i;
        printf("execv(\"%s\"", wanted);
        for (i = 0; argv[i] != NULL; i++)
            printf(", \"%s\"", argv[i]);
        printf(")\n");
    }
    execv(wanted, argv);
    fprintf(stderr, "Exec of %s failed\n", wanted);
    exit(1);
}

/* ibm@88699
 * "Borrowed" from Solaris 10 where the unsetenv() function is being added
 * to libc thanks to SUSv3 (Standard Unix Specification, version 3). As
 * such, in the fullness of time this will appear in libc on all relevant
 * Solaris/Linux platforms and maybe even the Windows platform.  At that
 * time, this stub can be removed.
 *
 * This implementation removes the environment locking for multithreaded
 * applications.  (We don't have access to these mutexes within libc and
 * the launcher isn't multithreaded.)  Note that what remains is platform
 * independent, because it only relies on attributes that a POSIX environment
 * defines.
 *
 * Returns 0 on success, -1 on failure.
 *
 * Also removed was the setting of errno.  The only value of errno set
 * was EINVAL ("Invalid Argument").
 */

/*
 * s1(environ) is name=value
 * s2(name) is name(not the form of name=value).
 * if names match, return value of 1, else return 0
 */
static int
match_noeq(const char *s1, const char *s2)
{
	while (*s1 == *s2++) {
		if (*s1++ == '=')
			return (1);
	}
	if (*s1 == '=' && s2[-1] == '\0')
		return (1);
	return (0);
}

/* ibm@88699
 * added for SUSv3 standard
 *
 * Delete entry from environ.
 * Do not free() memory!  Other threads may be using it.
 * Keep it around forever.
 */
static int
borrowed_unsetenv(const char *name)
{
	long	idx;		/* index into environ */

	if (name == NULL || *name == '\0' ||
	    strchr(name, '=') != NULL) {
		return (-1);
	}

	for (idx = 0; environ[idx] != NULL; idx++) {
		if (match_noeq(environ[idx], name))
			break;
	}
	if (environ[idx] == NULL) {
		/* name not found but still a success */
		return (0);
	}
	/* squeeze up one entry */
	do {
		environ[idx] = environ[idx+1];
	} while (environ[++idx] != NULL);

	return (0);
}
/* --- End of "borrowed" code --- */

/* ibm@88699
 * Wrapper for unsetenv() function.
 */
int
UnsetEnv(char *name)
{
    return(borrowed_unsetenv(name));
}

/*  ibm@88699
 *	Since using the file system as a registry is a bit risky, perform
 *	additional sanity checks on the identified directory to validate
 *	it as a valid jre/sdk.
 *
 *	Return 0 if the tests fail; otherwise return non-zero (true).
 *
 *	Note that checking for anything more than the existence of an
 *	executable object at bin/java relative to the path being checked
 *	will break the regression tests.
 *
 *  ibm@91094
 *      Without checking for more than bin/java we won't detect IBM JRE
 *      installations, which do not install a softlink to jre/bin/java.
 */
static int
CheckSanity(char *path, char *dir)
{
    char    buffer[PATH_MAX];

    if (strlen(path) + strlen(dir) + 15 > PATH_MAX)
	return (0);	/* Silently reject "impossibly" long paths */

    (void)strcat(strcat(strcat(strcpy(buffer, path), "/"), dir), "/bin/java");
    /*ibm@91094..*/
    if (access(buffer, X_OK) == 0) {
        return 1;
    } else {
        (void)strcat(strcat(strcat(strcpy(buffer, path), "/"), dir),
            "/jre/bin/java");
        return ((access(buffer, X_OK) == 0) ? 1 : 0);
    }
    /*..ibm@91094*/
}

/* ibm@88699
 * Find a command in a directory, returning the path.
 */
static char *
Resolve(char *indir, char *cmd)
{
    char name[PATH_MAX + 2], *real;

    if ((strlen(indir) + strlen(cmd) + 1)  > PATH_MAX) return 0;
    sprintf(name, "%s%c%s", indir, FILE_SEPARATOR, cmd);
    if (!ProgramExists(name)) return 0;
    real = MemAlloc(PATH_MAX + 2);
    if (!realpath(name, real)) 
	strcpy(real, name);
    return real;
}

/* ibm@88699
 * Find a path for the executable
 */
static char *
FindExecName(char *program)
{
    char cwdbuf[PATH_MAX+2];
    char *path;
    char *tmp_path;
    char *f;
    char *result = NULL;

    /* absolute path? */
    if (*program == FILE_SEPARATOR || 
	(FILE_SEPARATOR=='\\' && strrchr(program, ':')))
	return Resolve("", program+1);

    /* relative path? */
    if (strrchr(program, FILE_SEPARATOR) != 0) {
	char buf[PATH_MAX+2];
	return Resolve(getcwd(cwdbuf, sizeof(cwdbuf)), program);
    }

    /* from search path? */
    path = getenv("PATH");
    if (!path || !*path) path = ".";
    tmp_path = MemAlloc(strlen(path) + 2);
    strcpy(tmp_path, path);

    for (f=tmp_path; *f && result==0; ) {
	char *s = f;
	while (*f && (*f != PATH_SEPARATOR)) ++f;
	if (*f) *f++ = 0;
	if (*s == FILE_SEPARATOR)
	    result = Resolve(s, program);
	else {
	    /* relative path element */
	    char dir[2*PATH_MAX];
	    sprintf(dir, "%s%c%s", getcwd(cwdbuf, sizeof(cwdbuf)), 
		    FILE_SEPARATOR, s);
	    result = Resolve(dir, program);
	}
	if (result != 0) break;
    }

    free(tmp_path);
    return result;
}

/* Store the name of the executable once computed */
static char *execname = NULL;

/* ibm@88699
 * Compute the name of the executable
 *
 * In order to re-exec securely we need the absolute path of the
 * executable. On Solaris getexecname(3c) may not return an absolute
 * path so we use dladdr to get the filename of the executable and
 * then use realpath to derive an absolute path. From Solaris 9
 * onwards the filename returned in DL_info structure from dladdr is
 * an absolute pathname so technically realpath isn't required.
 * On Linux we read the executable name from /proc/self/exe.
 * As a fallback, and for platforms other than Solaris and Linux,
 * we use FindExecName to compute the executable name.
 */
static char *
SetExecname(char **argv)
{
    char* exec_path = NULL;

    if (execname != NULL)	/* Already determined */
	return (execname);
   
#if defined(__sun)
    {
        Dl_info dlinfo;
        if (dladdr((void*)&SetExecname, &dlinfo)) {
	    char *resolved = (char*)MemAlloc(PATH_MAX+1);
   	    if (resolved != NULL) {
		exec_path = realpath(dlinfo.dli_fname, resolved);
		if (exec_path == NULL) {
		    free(resolved);
		}
	    }
        }
    }
#elif defined(__linux__)
    {
	const char* self = "/proc/self/exe";
        char buf[PATH_MAX+1];
        int len = readlink(self, buf, PATH_MAX);
        if (len >= 0) {
	    buf[len] = '\0';		/* readlink doesn't nul terminate */
	    exec_path = strdup(buf);
	}
    }
#else /* !__sun && !__linux */
    {
        /* Not implemented */
    }
#endif 

    if (exec_path == NULL) {
        exec_path = FindExecName(argv[0]);
    }
    execname = exec_path;
    return exec_path;
}

/* ibm@88699
 * Return true if the named program exists
 */
static int
ProgramExists(char *name)
{
    struct stat sb;
    if (stat(name, &sb) != 0) return 0;
    if (S_ISDIR(sb.st_mode)) return 0;
    return (sb.st_mode & S_IEXEC) != 0;
}

void ReportErrorMessage(char * message, jboolean always) {
  if (always) {
    fprintf(stderr, "%s\n", message);
  }
}

void ReportErrorMessage2(char * format, char * string, jboolean always) {
  if (always) {
    fprintf(stderr, format, string);
    fprintf(stderr, "\n");
  }
}

void  ReportExceptionDescription(JNIEnv * env) {
  (*env)->ExceptionDescribe(env);
}

jboolean RemovableMachineDependentOption(char * option) {
  /*
   * Unconditionally remove both -d32 and -d64 options since only
   * the last such options has an effect; e.g. 
   * java -d32 -d64 -d32 -version
   * is equivalent to 
   * java -d32 -version
   */

  if( (strcmp(option, "-d32")  == 0 ) || 
      (strcmp(option, "-d64")  == 0 ))
    return JNI_TRUE;
  else
    return JNI_FALSE;
}

/*ibm@88699 end*/
